@@ -11,7 +11,9 @@ import fdasrsf.fPCA as fpca
 import fdasrsf.geometry as geo
 from scipy.integrate import trapz, cumtrapz
 from scipy.interpolate import interp1d
-from scipy.linalg import svd
+from scipy.linalg import svd, cholesky
+from scipy.cluster.hierarchy import linkage, fcluster
+from scipy.spatial.distance import squareform, pdist
 from numpy.linalg import norm
 from numpy.random import rand, normal
 from joblib import Parallel, delayed
@@ -71,14 +73,17 @@ class fdawarp:
         self.rsamps = False
     
 
-    def srsf_align(self, method="mean", omethod="DP2", smoothdata=False, parallel=False, lam=0.0, cores=-1, grid_dim=7):
+    def srsf_align(self, method="mean", omethod="DP2", smoothdata=False, MaxItr=20,
+                   parallel=False, lam=0.0, cores=-1, grid_dim=7):
         """
         This function aligns a collection of functions using the elastic
         square-root slope (srsf) framework.
 
-        :param method: (string) warp calculate Karcher Mean or Median (options = "mean" or "median") (default="mean")
+        :param method: (string) warp calculate Karcher Mean or Median 
+                       (options = "mean" or "median") (default="mean")
         :param omethod: optimization method (DP, DP2, RBFGS) (default = DP2)
         :param smoothdata: Smooth the data using a box filter (default = F)
+        :param MaxItr: Maximum number of iterations (default = 20)
         :param parallel: run in parallel (default = F)
         :param lam: controls the elasticity (default = 0)
         :param cores: number of cores for parallel (default = -1 (all))
@@ -153,7 +158,6 @@ class fdawarp:
         if method == 1:
             print("Compute Karcher Median of %d function in SRSF space..." % N)
 
-        MaxItr = 20
         ds = np.repeat(0.0, MaxItr + 2)
         ds[0] = np.inf
         qun = np.repeat(0.0, MaxItr + 1)
@@ -487,7 +491,8 @@ class fdawarp:
 
         return
 
-    def multiple_align_functions(self, mu, omethod="DP2", smoothdata=False, parallel=False, lam=0.0, cores=-1, grid_dim=7):
+    def multiple_align_functions(self, mu, omethod="DP2", smoothdata=False,
+                                 parallel=False, lam=0.0, cores=-1, grid_dim=7):
         """
         This function aligns a collection of functions using the elastic square-root
         slope (srsf) framework.
@@ -572,6 +577,40 @@ class fdawarp:
         return
 
 
+def pairwise_align_functions(f1, f2, time, omethod="DP2", lam=0, grid_dim=7):
+    """
+    This function aligns f2 to f1 using the elastic square-root
+        slope (srsf) framework.
+
+    Usage:  out = pairwise_align_functions(f1, f2, time)
+            out = pairwise_align_functions(f1, f2, time, omethod, lam, grid_dim)
+    
+    :param f1: vector defining M samples of function 1
+    :param f2: vector defining M samples of function 2
+    :param time: time vector of length M
+    :param omethod: optimization method (DP, DP2, RBFGS) (default = DP)
+    :param lam: controls the elasticity (default = 0)
+    :param grid_dim: size of the grid, for the DP2 method only (default = 7)
+
+    :rtype list containing
+    :return f2n: aligned f2
+    :return gam: warping function
+    :return q2n: aligned q2 (srsf)
+
+    """
+
+    q1 = uf.f_to_srsf(f1, time)
+    q2 = uf.f_to_srsf(f2, time)
+
+    gam = uf.optimum_reparam(q1, time, q1, omethod, lam, grid_dim)
+
+    f2n = uf.warp_f_gamma(time, f2 , gam)
+    q2n = uf.f_to_srsf(f2n, time)
+
+
+    return (f2n, gam, q2n)
+
+
 def pairwise_align_bayes(f1i, f2i, time, mcmcopts=None):
     """
     This function aligns two functions using Bayesian framework. It will align
